@@ -68,7 +68,7 @@ EBImoran.mc <- function (n, x, listw, nsim, zero.policy = FALSE,
     lres
 }
 
-probmap <- function(n, x) {
+probmap <- function(n, x, row.names=NULL) {
     if (!is.numeric(x)) 
         stop(paste(deparse(substitute(x)), "is not a numeric vector"))
     if (!is.numeric(n)) 
@@ -88,7 +88,12 @@ probmap <- function(n, x) {
     expCount <- x*b
     relRisk <- 100*(n/expCount)
     pmap <- ppois(n, expCount)
-    res <- data.frame(raw=p, expCount=expCount, relRisk=relRisk, pmap=pmap)
+    if (is.null(row.names)) 
+	res <- data.frame(raw=p, expCount=expCount, relRisk=relRisk, 
+	pmap=pmap)
+    else
+    	res <- data.frame(raw=p, expCount=expCount, relRisk=relRisk, 
+	pmap=pmap, row.names=row.names)
     res
 }
 
